@@ -4,7 +4,7 @@
  * GChemPaint library
  * reaction-arrow.cc 
  *
- * Copyright (C) 2004-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -36,6 +36,7 @@
 #include <gccv/arrow.h>
 #include <gccv/canvas.h>
 #include <gccv/group.h>
+#include <gcugtk/ui-manager.h>
 #include <gcu/objprops.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
@@ -105,6 +106,7 @@ bool ReactionArrow::Load (xmlNodePtr node)
 {
 	char *buf;
 	Object *parent, *prop;
+	gcu::Document *doc = GetDocument ();
 	xmlNodePtr child;
 	if (Arrow::Load (node)) {
 		buf = (char*) xmlGetProp (node, (xmlChar*) "type");
@@ -136,26 +138,27 @@ bool ReactionArrow::Load (xmlNodePtr node)
 			return true;
 		buf = (char*) xmlGetProp (node, (xmlChar*) "start");
 		if (buf) {
-			m_Start = reinterpret_cast<ReactionStep*> (parent->GetDescendant (buf));
+			doc->SetTarget (buf, reinterpret_cast <Object **> (&m_Start), GetParent (), this, ActionIgnore);
 			xmlFree (buf);
-			if (!m_Start)
-				return false;
-			m_Start->AddArrow (this);
 		}
 		buf = (char*) xmlGetProp (node, (xmlChar*) "end");
 		if (buf) {
-			m_End = reinterpret_cast<ReactionStep*> (parent->GetDescendant (buf));
+			doc->SetTarget (buf, reinterpret_cast <Object **> (&m_End), GetParent (), this, ActionIgnore);
 			xmlFree (buf);
-			if (!m_End)
-				return false;
-			m_End->AddArrow (this);
 		}
 		return true;
 	}
-puts("1");
 	return false;
 }
 
+void ReactionArrow::OnLoaded ()
+{
+	if (m_Start)
+		m_Start->AddArrow (this);
+	if (m_End)
+		m_End->AddArrow (this);
+}
+
 void ReactionArrow::AddItem ()
 {
 	if (m_Item)
@@ -272,14 +275,15 @@ static void do_free_data (struct CallbackData *data)
 	delete data;
 }
 
-bool ReactionArrow::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y)
+bool ReactionArrow::BuildContextualMenu (gcu::UIManager *UIManager, Object *object, double x, double y)
 {
+	GtkUIManager *uim = static_cast < gcugtk::UIManager * > (UIManager)->GetUIManager ();
 	Document *Doc = dynamic_cast<Document*> (GetDocument ());
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (Doc->GetWidget ()), "data");
 	// Don't allow more than one child at the moment
 	if (pData->SelectedObjects.size () != 1 || HasChildren ())
 		return Object::BuildContextualMenu (UIManager, object, x, y);
-	Object *obj = pData->SelectedObjects.front ();
+	Object *obj = *pData->SelectedObjects.begin ();
 	TypeId Id = obj->GetType ();
 	if ((Id != MoleculeType && Id != TextType) || obj->GetGroup ())
 		return Object::BuildContextualMenu (UIManager, object, x, y);
@@ -295,8 +299,8 @@ bool ReactionArrow::BuildContextualMenu (GtkUIManager *UIManager, Object *object
 	g_signal_connect_swapped (action, "activate", G_CALLBACK (do_attach_object), data);
 	gtk_action_group_add_action (group, action);
 	g_object_unref (action);
-	gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Arrow'><menuitem action='attach'/></menu></popup></ui>", -1, NULL);
-	gtk_ui_manager_insert_action_group (UIManager, group, 0);
+	gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Arrow'><menuitem action='attach'/></menu></popup></ui>", -1, NULL);
+	gtk_ui_manager_insert_action_group (uim, group, 0);
 	g_object_unref (group);
 	return true;
 }
