@@ -4,7 +4,7 @@
  * Gnome Crystal
  * atomsdlg.cc 
  *
- * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -104,7 +104,7 @@ static void on_charge_changed (GtkSpinButton *btn, AtomsDlg *pBox)
 	pBox->SetCharge (gtk_spin_button_get_value_as_int (btn));
 }
 
-AtomsDlg::AtomsDlg (Application *App, Document* pDoc): Dialog (App, UIDIR"/atoms.ui", "atoms", GETTEXT_PACKAGE, pDoc)
+AtomsDlg::AtomsDlg (Application *App, Document* pDoc): gcugtk::Dialog (App, UIDIR"/atoms.ui", "atoms", GETTEXT_PACKAGE, pDoc)
 {
 	m_pDoc = pDoc;
 	GtkWidget *frame = GetWidget ("mendeleiev");
@@ -236,7 +236,7 @@ bool AtomsDlg::Apply ()
 			g_array_index (m_Atoms, struct AtomStruct, m_AtomSelected).Alpha = gtk_color_button_get_alpha (AtomColor) / 65535.;
 		} else
 			g_array_index (m_Atoms, struct AtomStruct, m_AtomSelected).CustomColor = false;
-		if ((!GetNumber (AtomR, &(m_Radius.value.value), Min, 0)) || (m_Radius.value.value == 0.0)) {
+		if ((!GetNumber (AtomR, &(m_Radius.value.value), gcugtk::Min, 0)) || (m_Radius.value.value == 0.0)) {
 		} else
 			g_array_index (m_Atoms, struct AtomStruct, m_AtomSelected).Radius = m_Radius;
 		g_array_index (m_Atoms, struct AtomStruct, m_AtomSelected).EffectiveRadiusRatio = gtk_spin_button_get_value (ScaleBtn) / 100.;
@@ -364,7 +364,7 @@ void AtomsDlg::AtomSelect(GtkTreeSelection *Selection)
 			g_array_index (m_Atoms, struct AtomStruct, m_AtomSelected).Alpha = gtk_color_button_get_alpha (AtomColor) / 65535.;
 		} else
 			g_array_index (m_Atoms, struct AtomStruct, m_AtomSelected).CustomColor = false;
-		if ((!GetNumber (AtomR, &(m_Radius.value.value), Min, 0)) || (m_Radius.value.value == 0.0)) {
+		if ((!GetNumber (AtomR, &(m_Radius.value.value), gcugtk::Min, 0)) || (m_Radius.value.value == 0.0)) {
 		} else
 			g_array_index(m_Atoms, struct AtomStruct, m_AtomSelected).Radius = m_Radius;
 		g_array_index (m_Atoms, struct AtomStruct, m_AtomSelected).EffectiveRadiusRatio = gtk_spin_button_get_value (ScaleBtn) / 100.;
@@ -602,8 +602,14 @@ void AtomsDlg::PopulateRadiiMenu ()
 	const GcuAtomicRadius **radius = m_Radii;
 	int i = m_RadiiIndex.size () - 2;
 	g_signal_handler_block (RadiusMenu, m_RadiiSignalID);
+#if GTK_CHECK_VERSION (2, 24, 0)
+	GtkListStore *list = GTK_LIST_STORE (gtk_combo_box_get_model (RadiusMenu));
+	GtkTreeIter iter;
+	gtk_list_store_clear (list);
+#else
 	for (int j = 0; j <= i; j++)
 		gtk_combo_box_remove_text (RadiusMenu, 1);
+#endif
 	m_RadiiIndex.clear ();
 	string str;
 	m_RadiiIndex.push_back (-1);
@@ -627,7 +633,12 @@ void AtomsDlg::PopulateRadiiMenu ()
 				str += string(" (") + string(((*radius)->spin == GCU_LOW_SPIN)? _("low spin"): _("high spin")) + string(")");
 			if (!str.length())
 				str = _("Database");
+#if GTK_CHECK_VERSION (2, 24, 0)
+			gtk_list_store_append (list, &iter);
+			gtk_list_store_set (list, &iter, 0, str.c_str (), -1);
+#else
 			gtk_combo_box_append_text (RadiusMenu, str.c_str ());
+#endif
 			m_RadiiIndex.push_back(i++);
 			radius++;
 		}
