@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * libs/gcu/document.cc
  *
- * Copyright (C) 2004-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -54,9 +54,10 @@ Document::~Document ()
 char* Document::GetNewId (char const *id, bool Cache)
 {
 	gchar *Id = g_strdup (id);
-	int i = 0;
+	int i = 0, k;
 	while ((Id[i] < '0') || (Id[i] > '9'))
 		i++;
+	k = atoi (id + i);
 	gchar *buf = new gchar[i + 16];
 	strncpy (buf, Id, i);
 	buf[i] = 0;
@@ -66,22 +67,17 @@ char* Document::GetNewId (char const *id, bool Cache)
 	if (s.size ())
 		j = atoi (s.c_str ());
 	char* key = g_strdup (buf);
-	while (snprintf (buf + i, 16, "%d", j++), GetDescendant (buf) != NULL) ;
+	while (snprintf (buf + i, 16, "%d", j), GetDescendant (buf) != NULL)
+		j++;
 	Id = g_strdup_printf ("%d", j);
-	if (Cache) {
-		m_TranslationTable[key] = Id;
-		m_TranslationTable[id] = buf;
-	}
+	Object *obj = GetDescendant (id);
+	if (obj && (k > 1 || m_NewObjects.find (obj) == m_NewObjects.end ()))
+		if (Cache) {
+			m_TranslationTable[key] = Id;
+			m_TranslationTable[id] = buf;
+		}
 	g_free (Id);
 	g_free (key);
-	if (m_PendingTable.size () > 0) {
-		std::map <std::string, list <PendingTarget> >::iterator it, end = m_PendingTable.end ();
-		if ((it = m_PendingTable.find (id)) != end) {
-			// Hmm, this might be unsecure if several imported objects have the same Id
-			m_PendingTable[buf] = (*it).second;
-			m_PendingTable.erase (it);
-		}
-	}
 	return buf;
 }
 
@@ -158,6 +154,8 @@ bool Document::Loaded () throw (LoaderError)
 	std::set <Object *>::iterator k, kend = m_DirtyObjects.end ();
 	for (k = m_DirtyObjects.begin (); k != kend; k++)
 		(*k)->OnLoaded ();
+	m_DirtyObjects.clear ();
+	m_TranslationTable.clear ();
 	return count > 0;
 }
 
