@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * molecule.cc 
  *
- * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -32,6 +32,7 @@
 #include "document.h"
 #include "formula.h"
 #include "residue.h"
+#include <gsf/gsf-output-memory.h>
 #include <glib/gi18n-lib.h>
 #include <stack>
 
@@ -40,14 +41,16 @@ using namespace std;
 namespace gcu
 {
 
-Molecule::Molecule (TypeId Type): Object (Type)
+Molecule::Molecule (TypeId Type, ContentType ct): Object (Type)
 {
 	SetId ("m1");
+	m_Content = ct;
 }
 
-Molecule::Molecule (Atom* pAtom): Object (MoleculeType)
+Molecule::Molecule (Atom* pAtom, ContentType ct): Object (MoleculeType)
 {
 	SetId ("m1");
+	m_Content = ct;
 	SetParent (pAtom->GetDocument ());
 	AddAtom (pAtom);
 	Chain* pChain = new Chain (this, pAtom); //will find the cycles
@@ -88,6 +91,8 @@ void Molecule::AddChild (Object* object)
 		AddBond (bond);
 		break;
 	}
+	default:
+		Object::AddChild (object);
 	}
 }
 
@@ -390,4 +395,82 @@ std::string Molecule::Name ()
 	return _("Molecule");
 }
 
+std::string const &Molecule::GetCML ()
+{
+	if (m_CML.length () == 0) {
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->Save (output, "chemical/x-cml", this, m_Content);
+		size_t l = gsf_output_size (output);
+		if (l > 0)
+			m_CML.assign (reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output))), l);
+		g_object_unref (output);
+	}
+	return m_CML;
+}
+
+std::string const &Molecule::GetInChI ()
+{
+	if (m_InChI.length () == 0) {
+		if (m_CML.length () == 0)
+			GetCML ();
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->ConvertFromCML (m_CML.c_str (), output, "inchi");
+		size_t l = gsf_output_size (output);
+		if (l > 0) {
+			char const *res = reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output)));
+			while (res[l - 1] < ' ')
+				l--;
+			m_InChI.assign (res, l);
+		}
+		g_object_unref (output);
+	}
+	return m_InChI;
+}
+
+std::string const &Molecule::GetInChIKey ()
+{
+	if (m_InChIKey.length () == 0) {
+		if (m_CML.length () == 0)
+			GetCML ();
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->ConvertFromCML (m_CML.c_str (), output, "inchi", "-xK");
+		size_t l = gsf_output_size (output);
+		if (l > 0) {
+			char const *res = reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output)));
+			while (res[l - 1] < ' ')
+				l--;
+			m_InChIKey.assign (res, l);
+		}
+		g_object_unref (output);
+	}
+	return m_InChIKey;
+}
+
+std::string const &Molecule::GetSMILES ()
+{
+	if (m_SMILES.length () == 0) {
+		if (m_CML.length () == 0)
+			GetCML ();
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->ConvertFromCML (m_CML.c_str (), output, "can");
+		size_t l = gsf_output_size (output);
+		if (l > 0) {
+			char const *res = reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output)));
+			while (res[l - 1] < ' ')
+				l--;
+			m_SMILES.assign (res, l);
+		}
+		g_object_unref (output);
+	}
+	return m_SMILES;
+}
+
+void Molecule::ResetIndentifiers ()
+{
+	m_CML.clear ();
+	m_InChI.clear ();
+	m_InChIKey.clear ();
+	m_SMILES.clear ();
+}
+
 }	//namespace gcu
