@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * libs/gcu/molecule.h 
  *
- * Copyright (C) 2001-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,6 +26,7 @@
 #define GCU_MOLECULE_H
 
 #include "object.h"
+#include "structs.h"
 
 /*!\file*/
 namespace gcu {
@@ -41,6 +42,7 @@ Represents molecules.
 */
 class Molecule: public Object
 {
+friend class Chain;
 public:
 /*!
 @param Type the type id of a derived class
@@ -48,14 +50,14 @@ public:
 The constructor. The type id may be changed in derived classes, otherwise the
 argument is not needed, since the default value is enough.
 */
-	Molecule (TypeId Type = MoleculeType);
+	Molecule (TypeId Type = MoleculeType, ContentType ct = ContentType3D);
 /*!
 @param pAtom
 
 Builds a molecule from an atom. If the atom has bonds, the connectivity will
 be explored and all atoms and bonds found will be added to the molecule.
 */
-	Molecule (Atom* pAtom);
+	Molecule (Atom* pAtom, ContentType ct = ContentType3D);
 /*!
 The destructor.
 */
@@ -145,6 +147,36 @@ Clears cycles and chains and call gcu::Object::Clear().
 @return the next bond of the molecule or NULL if all bonds have been previously returned.
 */
 	Bond const *GetNextBond(std::list<Bond*>::const_iterator &i) const;
+/*!
+@return the localized object generic name.
+*/
+/*!
+@param name a name.
+@param convention a naming convention, might be NULL.
+
+Adds a molecule name following \a convention. Only one name can be stored for
+a given convention.
+*/
+	void SetName (char const *name, char const *convention);
+/*!
+@param convention a naming convention.
+
+@return the molecule name using \a convention if known.
+*/
+	char const *GetName (char const *convention = NULL);
+/*!
+@return the localized object generic name.
+*/
+	std::string Name ();
+
+	void ResetIndentifiers ();
+	std::string const &GetCML ();
+/*!
+@return the InChI.
+*/
+	std::string const &GetInChI ();
+	std::string const &GetInChIKey ();
+	std::string const &GetSMILES ();
 
 protected:
 /*!
@@ -163,6 +195,14 @@ The atoms in the molecule.
 The bonds in the molecule.
 */
 	std::list<Bond*> m_Bonds;
+
+private:
+	std::map <std::string, std::string> m_Names;
+	std::string m_CML;
+	std::string m_InChI;
+	std::string m_InChIKey;
+	std::string m_SMILES;
+	ContentType m_Content;
 };
 
 }	//	namespace gcu
