@@ -4,7 +4,7 @@
  * GChemPaint selection plugin
  * bracketstool.h 
  *
- * Copyright (C) 2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2007-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,19 +26,43 @@
 #define GCHEMPAINT_BRACKETS_TOOL_H
 
 #include <gcp/tool.h>
+#include <gcp/fontsel.h>
+#include <gccv/brackets.h>
+#include <gccv/structs.h>
+
+namespace gcp {
+	class Application;
+}
 
-class gcp::Application;
-	
 class gcpBracketsTool: public gcp::Tool
 {
 public:
 	gcpBracketsTool (gcp::Application* App);
 	virtual ~gcpBracketsTool ();
 
-	virtual bool OnClicked ();
-	virtual void OnDrag ();
-	virtual void OnRelease ();
+	bool OnClicked ();
+	void OnDrag ();
+	void OnRelease ();
 	char const *GetHelpTag () {return "brackets";}
+	GtkWidget *GetPropertyPage ();
+	void Activate ();
+	bool Evaluate ();
+
+	static void OnTypeChanged (GtkComboBox *box, gcpBracketsTool *tool);
+	static void OnUsedChanged (GtkComboBox *box, gcpBracketsTool *tool);
+	static void OnFontChanged (GcpFontSel *fontsel, gcpBracketsTool *tool);
+
+private:
+	gccv::BracketsTypes m_Type;
+	gccv::BracketsUses m_Used;
+	GcpFontSel *m_FontSel;
+	gccv::Rect m_ActualBounds;
+	gccv::Item *m_Rect, *m_Bracket;
+	std::string m_FontFamily;
+	int m_FontSize;
+	PangoFontDescription *m_FontDesc;
+	std::string m_FontName;
+	gcu::Object *m_Target;
 };
 
 #endif // GCHEMPAINT_BRACKETS_TOOL_H
