@@ -4,7 +4,7 @@
  * Gnome Crystal
  * cleavage.cc 
  *
- * Copyright (C) 2001-2004 Jean Brfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2010 Jean Brfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -27,32 +27,32 @@
 #include "cleavage.h"
 #include <glib.h>
 
-gcCleavage::gcCleavage(): CrystalCleavage()
+gcCleavage::gcCleavage (): gcr::Cleavage ()
 {
 }
 
-gcCleavage::~gcCleavage()
+gcCleavage::~gcCleavage ()
 {
 }
 
-bool gcCleavage::LoadOld(xmlNodePtr node)
+bool gcCleavage::LoadOld (xmlNodePtr node)
 {
 	char *txt;
-	txt = (char*)xmlGetProp(node, (xmlChar*)"indices");
-	if (!txt) return false;
-	if (sscanf(txt, "%d %d %d", &m_nh, &m_nk, &m_nl) != 3)
-	{
-		xmlFree(txt);
+	txt = (char*) xmlGetProp (node, (xmlChar*) "indices");
+	if (!txt)
+		return false;
+	if (sscanf (txt, "%d %d %d", &m_nh, &m_nk, &m_nl) != 3) {
+		xmlFree (txt);
 		return false;
 	}
-	xmlFree(txt);
-	txt = (char*)xmlGetProp(node, (xmlChar*)"planes");
-	if (!txt) return false;
-	if (sscanf(txt, "%d", &m_nPlanes)!= 1)
-	{
-		xmlFree(txt);
+	xmlFree (txt);
+	txt = (char*) xmlGetProp (node, (xmlChar*) "planes");
+	if (!txt)
+		return false;
+	if (sscanf (txt, "%d", &m_nPlanes)!= 1) {
+		xmlFree (txt);
 		return false;
 	}
-	xmlFree(txt);
+	xmlFree (txt);
 	return true;
 }
