@@ -4,7 +4,7 @@
  * Gnome Crystal
  * view-settings.cc
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -26,17 +26,15 @@
 #include "view-settings.h"
 #include "document.h"
 #include "application.h"
-#include <glade/glade.h>
 
-gcViewSettingsDlg::gcViewSettingsDlg (gcView* pView): Dialog (pView->GetDocument ()->GetApplication (), DATADIR"/gchemutils-unstable/glade/crystal/view-settings.glade", "view_settings")
+gcViewSettingsDlg::gcViewSettingsDlg (gcView* pView): gcugtk::Dialog (static_cast < gcugtk::Application * > (pView->GetDoc ()->GetApp ()), UIDIR"/view-settings.ui", "view_settings", GETTEXT_PACKAGE, pView)
 {
 	m_pView = pView;
-	m_pView->NotifyDialog (this);
-	FoV = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "fov"));
-	Psi = (GtkEntry *) glade_xml_get_widget (xml, "psi");
-	Theta = (GtkEntry *) glade_xml_get_widget (xml, "theta");
-	Phi = (GtkEntry *) glade_xml_get_widget (xml, "phi");
-	Background = (GtkColorButton *) glade_xml_get_widget (xml, "color");
+	FoV = GTK_SPIN_BUTTON (GetWidget ("fov"));
+	Psi = GTK_ENTRY (GetWidget ("psi"));
+	Theta = GTK_ENTRY (GetWidget ("theta"));
+	Phi = GTK_ENTRY (GetWidget ("phi"));
+	Background = GTK_COLOR_BUTTON (GetWidget ("color"));
 	double x0, x1, x2, x3;
 	GdkColor color;
 	m_pView->GetBackgroundColor (&x0, &x1, &x2, &x3);
@@ -60,17 +58,16 @@ gcViewSettingsDlg::gcViewSettingsDlg (gcView* pView): Dialog (pView->GetDocument
 
 gcViewSettingsDlg::~gcViewSettingsDlg ()
 {
-	m_pView->RemoveDialog (this);
 }
 
 bool gcViewSettingsDlg::Apply()
 {
 	double x0, x1, x2;
-	if (!GetNumber (Psi, &x0, MinEqMax, -180, 180))
+	if (!GetNumber (Psi, &x0, gcugtk::MinEqMax, -180, 180))
 		return false;
-	if (!GetNumber (Theta, &x1, MinEqMaxEq, 0, 180))
+	if (!GetNumber (Theta, &x1, gcugtk::MinEqMaxEq, 0, 180))
 		return false;
-	if (!GetNumber (Phi, &x2, MinEqMax, -180, 180))
+	if (!GetNumber (Phi, &x2, gcugtk::MinEqMax, -180, 180))
 		return false;
 	m_pView->SetRotation (x0, x1, x2);
 	m_pView->GetFoV () = gtk_spin_button_get_value (FoV);
@@ -78,6 +75,6 @@ bool gcViewSettingsDlg::Apply()
 	gtk_color_button_get_color (Background, &color);
 	m_pView->SetBackgroundColor (color.red / 65535., color.green / 65535., color.blue / 65535., gtk_color_button_get_alpha (Background) / 65535.);
 	m_pView->Update ();
-	m_pView->GetDocument ()->SetDirty ();
+	dynamic_cast <gcDocument *> (m_pView->GetDoc ())->SetDirty (true);
 	return true;	
 }
