@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * programs/spectra/gspectrum.cc
  *
- * Copyright (C) 2007-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2007-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -51,8 +51,6 @@ int main (int argc, char *argv[])
 
 	// create a new Application. This initialize Goffice
 	gsvApplication *App = new gsvApplication ();
-	// Initialize plugins manager
-	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_TYPE_PLUGIN_LOADER_MODULE);
 	gsvDocument *pDoc = App->OnFileNew();
 	char *path, *uri;
 	bool bres = false;
